/**
   Bojan Nikolic <b.nikolic@mrao.cam.ac.uk>, <bojan@bnikolic.co.uk>
   Initial version November 2009

   \file wvrretrieve.cpp

*/

#include <iostream>
#include <cmath>

#include <boost/program_options.hpp>

#include "../config.h"
#include "apps/almaabs.hpp"
#include "apps/almaresults.hpp"
#include "apps/almaopts.hpp"
#include "radiometermeasure.hpp"

void version(void)
{
  std::cout<<("Program wvrretrieve, part of LibAIR version "+std::string(PACKAGE_VERSION))
	   <<std::endl;
}

void help(const boost::program_options::options_description &desc)
{
  std::cout<<"Make a retrieval of water vapour and correction coefficients from WVR data"
	   <<std::endl
	   <<std::endl
	   <<"This program is part of LibAIR version " + std::string(VERSION)
	   <<std::endl
	   <<"GPL license -- you have the right to the source code. See COPYING"
	   <<std::endl
	   <<std::endl
	   <<"You should supply the observed WVR temperatures as four positional arguments:"
	   <<std::endl
	   <<"e.g.:"
	   <<std::endl
	   <<std::endl
	   <<"./wvrretrieve <Ch1Temp> <Ch2Temp> <Ch3Temp> <Ch4Temp>"
	   <<std::endl
	   <<"where channels are numbered from line centre out"
	   <<std::endl
	   <<std::endl
	   <<desc;
}

int main(int ac, char* av[])
{
  using namespace boost::program_options;

  options_description desc("Allowed options");
  desc.add_options()
    ("version",
     "Print the name and version of the program")
    ("help",
     "Print (this) help message")
    ("wvrtemp",
     value< std::vector<double> >(),
     "Temperature recorded by WVR, from innermost channel outward")
    ("elevation",
     value<double>()->default_value(90),
     "Elevation of observation in degrees")
    ("cont",
     "Try to constrain the continuum")
    ("osf",
     "Apply priors suitable for the OSF (if this option is not use then priors are for AOS)")
     ;

  positional_options_description p;
  p.add("wvrtemp", 4);


  variables_map vm;        
  store(command_line_parser(ac, av).
	options(desc).positional(p).run(),
	vm);
  notify(vm);    

  if (vm.count("version"))
  {
    version();
  }
  else if (vm.count("help") or vm["wvrtemp"].as< std::vector<double> >().size() != 4)
  {
    help(desc);
  }
  else
  {
    LibAIR::ALMAWVRCharacter WVRChar;

    if (vm.count("cont"))
    {
      LibAIR::ALMARetOpts opts;
      if (vm.count("osf"))
      {
	opts.OSFPriors=true;
      }

      LibAIR::ALMAContRes res;
      ALMAAbsContRetrieve(vm["wvrtemp"].as< std::vector<double> >(),
			  vm["elevation"].as<double>() * M_PI/180.0,
			  WVRChar,
			  res,
			  opts);

      std::cout<<res;

    }
    else 
    {
      if (vm.count("osf"))
      {
	std::cout<<"ERROR: --osf option is only understood by the continuum retrieval version of this program"
		 <<std::endl
		 <<"This restriction will be removed in future versions"
		 <<std::endl
		 <<"For the time being, to continue you, you must remove --osf"
		 <<std::endl;
	return -1;
      }
      LibAIR::ALMAAbsRet ar(vm["wvrtemp"].as< std::vector<double> >(),
			    vm["elevation"].as<double>() * M_PI/180.0,
			    WVRChar);
      
      LibAIR::ALMAResBase res;
      ar.g_Res(res);
      std::cout<<res;
    }
  }


    
}
