/**
   \file t_simple.cpp
   
   Bojan Nikolic <b.nikolic@mrao.cam.ac.uk>, <bojan@bnikolic.co.uk>
   Initial version April 2008

   Simple unit tests using the Boost framework
*/

#define BOOST_AUTO_TEST_MAIN

#include <boost/test/unit_test.hpp>
#include <boost/test/auto_unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include "minim.hxx"
#include "minimmodel.hxx"

#include "basicphys.hpp"
#include "slice.hpp"
#include "models_basic.hpp"
#include "radiometermeasure.hpp"
#include "columns.hpp"
#include "partitionsum.hpp"
#include "partitionsum_testdata.hpp"
#include "lineparams.hpp"
#include "layers.hpp"
#include "rtranfer.hpp"
#include "tbutils.hpp"

#include "columns_data.hpp"
#include "singlelayerwater.hpp"
#include "cloudywater.hpp"
#include "model_water.hpp"
#include "model_make.hpp"
#include "taumodel.hpp"

#include "dispersion.hpp"
#include "apps/antennautils.hpp"
#include "apps/dtdlcoeffs.hpp"

#include "numalgo.hpp"

BOOST_AUTO_TEST_CASE( Water22Data )
{
  using namespace LibAIR;
  boost::shared_ptr<HITRAN_entry> e ( Mk22WaterEntry() );
  BOOST_CHECK_EQUAL(e->S, 1.317288e-023);

}

BOOST_AUTO_TEST_CASE( WaterLinePars )
{
  using namespace LibAIR;
  boost::shared_ptr<HITRAN_entry> e ( Mk183WaterEntry() );
  PartitionTable  pt( getH2ORawTable() );
  CLineParams res;


  ComputeLineParsWQ(*e,
		    pt,
		    270,
		    550,
		    0,
		    296,
		    res);

  // Checked against AM
  BOOST_CHECK_EQUAL( res.gamma,
		     1.7187356542738073);

  // Checked against AM
  BOOST_CHECK_EQUAL( res.S,
		     2.7499585535915401e-21);

  // Checked against AM
  BOOST_CHECK_EQUAL( res.f0,
		     183.28244309999999);

  
}

BOOST_AUTO_TEST_CASE( WaterCont )
{
  using namespace LibAIR;

  ContinuumColumn cc( 1.0 * pmw_mm_to_n,
		      MkWaterGrossCont() );

  Slice sl (270,550);
  
  std::vector<double> f (1), res(1);
  f[0] = 183.3;
  cc.ComputeTau(f, sl, res);

  // Checked against AM
  BOOST_CHECK_CLOSE( res[0],
		     0.013293352418453331,
		     1e-10);  
  
}

BOOST_AUTO_TEST_CASE(Water183DataBasic)
{
  using namespace LibAIR;
  {
    Water183Data wd( PartNone, NoCont,1);
    BOOST_CHECK( not wd.wcont());
  }

  {
    Water183Data wd( PartNone, AirCont,1);
    BOOST_CHECK( wd.wcont());
  }
}

BOOST_AUTO_TEST_CASE( NonDispersiveConv )
{
  using namespace LibAIR;
  const double watertopath=SW_WaterToPath_Simplified(1.0, 273);
  
  BOOST_CHECK_MESSAGE( watertopath < 6.4 && watertopath > 6.2,
		       "Water paths is "<< watertopath);
}

BOOST_AUTO_TEST_CASE( SliceBasics )
{
  using namespace LibAIR;
  const double T =270;
  const double P =550;

  std::vector<double> f(1), res(1);
  f[0]=180;

  Slice sl (T,P);
  BOOST_CHECK( sl.getT() == T);
  BOOST_CHECK( sl.getP() == P);

  sl.ComputeTx(f, res);
  BOOST_CHECK(res[0] == 1.0);

}


BOOST_AUTO_TEST_CASE(Layer_CheckNumberOf)
{
  using namespace LibAIR;  
  IsoTLayer l ( 275, 600, 50);

  BOOST_CHECK_EQUAL( l.getNear().getP(),
		     600);

  BOOST_CHECK_EQUAL( l.getEnd() - l.getFar(),
		     11);
}

BOOST_AUTO_TEST_CASE(Layer_CheckSet)
{
  using namespace LibAIR;  
  IsoTLayer l ( 275, 600, 50);

  l.setT(270);

  BOOST_CHECK_EQUAL( l.getNear().getT(),
		     270);

  BOOST_CHECK_EQUAL( (*(l.getFar()++))->getT(),
		     270);

  l.setP(500);
  BOOST_CHECK_EQUAL( l.getNear().getP(),
		     500);

  BOOST_CHECK_EQUAL( (*l.getFar())->getP(),
		     450);
}

BOOST_AUTO_TEST_CASE(LayerBasic)
{
  using namespace LibAIR;  
  
  IsoTLayer l ( 275, 600, 1);
  
  boost::shared_ptr<HITRAN_entry> e ( Mk183WaterEntry() );
  PartitionTable  pt( getH2ORawTable() );
  TrivialGrossColumn h2ocol ( *e, &pt, 
			      1.0 * pmw_mm_to_n);

  l.AddColumn(h2ocol);

  std::vector<double> f(1), res(1);
  f[0]=180;
  LayerResult lr(l, f);
  OpaqueSlice os(2.7, 0);
  SliceResult osr(os, f);
  osr.UpdateI();
  lr.UpdateI(osr);

  BOOST_CHECK(   lr.UpdateTb()[0] >  1);

  Slice normals(275, 600);
  normals.AddColumn(h2ocol);
  SliceResult normals_res(normals, f);
  normals_res.UpdateI(osr);
  
  BOOST_CHECK_EQUAL(normals_res.UpdateTb()[0],
		    lr.UpdateTb()[0]);


  
}

BOOST_AUTO_TEST_CASE( WaterData_Constructs )
{
  using namespace LibAIR;
  WaterData wd(WaterData::L22,
	       PartTable,
	       AirCont,
	       1.0);
  
  BOOST_CHECK_EQUAL(wd.wcol()->getN(),
		    pmw_mm_to_n);
  
  Slice normals(275, 600);
  
  std::vector<double> f(1, 22.0);
  std::vector<double> res;
  wd.wcol()->ComputeTau(f,
			normals,
			res);
  BOOST_CHECK(res[0] > 0.001); 
}

BOOST_AUTO_TEST_CASE(MkSSBRadio_Flat_T)
{
  using namespace LibAIR;

  boost::scoped_ptr<Radiometer> r(MkSSBRadio(22.0, 1.0));
  
  std::vector<double> tsky(5, 1.0);
  
  BOOST_CHECK_EQUAL(r->eval(tsky, 0),
		    1.0);
  
}


BOOST_AUTO_TEST_CASE(MkIRAM22_constructs)
{
  using namespace LibAIR;
  
  boost::scoped_ptr<Radiometer> r(MkIRAM22());
  
  BOOST_CHECK_EQUAL(r->nchannels(),
		    3);

  BOOST_CHECK_EQUAL(r->getFGrid().size(),
		    15);

  BOOST_CHECK_CLOSE(r->getFGrid()[0],
		    19.2-0.5,
		    1);

  std::vector<double> tsky(r->getFGrid().size(), 1.0);
  
  for (size_t i =0 ; i< 3; ++i)
  {
    BOOST_CHECK_EQUAL(r->eval(tsky, i),
		      1.0);
  }
}

BOOST_AUTO_TEST_CASE(ISingleLayerWater_Value)
{
  using namespace LibAIR;

  std::vector<double> f(1, 183.3);
  std::vector<double> res;
  
  ISingleLayerWater sl(f);
  sl.n=1.0;
  sl.T=270;
  sl.P=600;
  res=sl.TbGrid();
  BOOST_CHECK(res[0] > 100); 

  sl.n=0;
  res=sl.TbGrid();
  BOOST_CHECK(res[0]< 10);
}

BOOST_AUTO_TEST_CASE(H2OCol_basic)
{
  using namespace LibAIR;
  PartitionTable  pt( getH2ORawTable());
  H2OCol hc(&pt);
  Slice normals(275, 600);
    
  std::vector<double> f(1, 183.3);
  std::vector<double> res;

  hc.ComputeTau(f, normals, res);
  BOOST_CHECK_EQUAL(res[0], 0);
  
  hc.setN(1e30);
  hc.ComputeTau(f, normals, res);
  BOOST_CHECK(res[0] >  10);
  
}
  
BOOST_AUTO_TEST_CASE(ICloudyWater_basic)
{
  std::vector<double> f(1, 183.3);
  LibAIR::ICloudyWater ic(f);
  ic.T=260;
  ic.P=500;
  ic.n=0;
  ic.tau183=0;
  ic.CT=260;

  BOOST_CHECK_CLOSE(ic.TbGrid()[0],
		    // Background
		    0.351,
		    1);
  ic.tau183=100;
  BOOST_CHECK_CLOSE(ic.TbGrid()[0],
		    LibAIR::PlanckToRJ(183.3,
				       260.0),
		    0.1);  

}


BOOST_AUTO_TEST_CASE(EmpContColumn_numbers)
{
  LibAIR::EmpContColumn c(1.0, 2.0);
  
  std::vector<double> f(1, 6.0);  
  std::vector<double> res(1);

  LibAIR::Slice normals(275, 600);
  c.ComputeTau(f, 
	       normals,
	       res);
  
  BOOST_CHECK_EQUAL(res[0],
		    9.0);
  
}



BOOST_AUTO_TEST_CASE(WaterModel_basics)
{
  using namespace LibAIR;

  boost::shared_ptr<Radiometer>        r(SwitchRadiometer(ALMAProd));
  boost::shared_ptr<ISingleLayerWater> sl(new ISingleLayerWater (r->getFGrid()));  

  LibAIR::WaterModel<ISingleLayerWater> m(r,
					  sl);
  
  Minim::ModelDesc md(m);
  
  md.getbyname("n")->setp(1.0);
  md.getbyname("T")->setp(272);
  md.getbyname("P")->setp(550);

  BOOST_CHECK_CLOSE(m.eval(0),
		    178.0,
		    5);

}

BOOST_AUTO_TEST_CASE(mkSingleLayerWater_test)
{
  using namespace LibAIR;
  boost::scoped_ptr<WaterModel<ISingleLayerWater> >
    m (mkSingleLayerWater(ALMAProd,
			  PartTable,
			  AirCont));

  Minim::ModelDesc md(*m);
  
  md.getbyname("n")->setp(1.0);
  md.getbyname("T")->setp(272);
  md.getbyname("P")->setp(550);

  BOOST_CHECK_CLOSE(m->eval(0),
		    178.0,
		    5);

  boost::scoped_ptr<WaterModel<ISingleLayerWater> >
    m2 (mkSimpleOffset(0.0,0.0));
  
  Minim::ModelDesc md2(*m2);
  
  md2.getbyname("n")->setp(1.0);
  md2.getbyname("T")->setp(272);
  md2.getbyname("P")->setp(550);

  BOOST_CHECK_EQUAL(m->eval(0),
		    m2->eval(0));

  
}

BOOST_AUTO_TEST_CASE(ALMAWVRCharacter_mk)
{

  using namespace LibAIR;

  boost::shared_ptr<Radiometer>        r(SwitchRadiometer(ALMAProd));

  LibAIR::ALMAWVRCharacter c;

  boost::shared_ptr<LibAIR::Radiometer> r2 (LibAIR::MkALMAWVR(c));


  boost::shared_ptr<ISingleLayerWater> sl(new ISingleLayerWater (r->getFGrid()));  
  boost::shared_ptr<ISingleLayerWater> sl2(new ISingleLayerWater (r2->getFGrid()));  

  LibAIR::WaterModel<ISingleLayerWater> m(r,
					  sl);

  LibAIR::WaterModel<ISingleLayerWater> m2(r2,
					   sl2);
  
  Minim::ModelDesc md(m);
  
  md.getbyname("n")->setp(1.0);
  md.getbyname("T")->setp(272);
  md.getbyname("P")->setp(550);

  Minim::ModelDesc md2(m2);
  
  md2.getbyname("n")->setp(1.0);
  md2.getbyname("T")->setp(272);
  md2.getbyname("P")->setp(550);

  for(size_t i =0; i<4; ++i)
  {
    BOOST_CHECK_CLOSE(m.eval(i),
		      m2.eval(i),
		      0.001);
  }
  

}

BOOST_AUTO_TEST_CASE(TauModel_basic)
{
  using namespace LibAIR;
  
  TauModel tm;
  
  BOOST_CHECK( tm.eval(0) > 0);

  Minim::ModelDesc md(tm);
  
  md.getbyname("n")->setp(0);
  
  BOOST_CHECK_CLOSE( tm.eval(0), 0.35186, 1);  

  md.getbyname("n")->setp(1);

  BOOST_CHECK( tm.eval(0) > 20);  
  
}

BOOST_AUTO_TEST_CASE(DispersionTab_basic)
{
  using namespace LibAIR;
  DispersionTab t;
  loadCSV("../data/libair-ddefault.csv", t);

  BOOST_CHECK_CLOSE( t(3.47e+11) , 0.137168,
		     0.0000000001);  
  BOOST_CHECK_CLOSE( t(3.474999e+11) , 0.138015,
		     0.001);  
}


BOOST_AUTO_TEST_CASE(nearestAnts_basic)
{
  using namespace LibAIR;
  antpos_t pos(5, 3);
  pos(0,0)=0;   pos(0,1)=0;   pos(0,2)=0;
  pos(1,0)=1;   pos(1,1)=0;   pos(1,2)=0;
  pos(2,0)=0;   pos(2,1)=1;   pos(2,2)=0;
  pos(3,0)=0;   pos(3,1)=0;   pos(3,2)=1;
  pos(4,0)=0.1;   pos(4,1)=0;   pos(4,2)=0;

  AntSet flag;
  {
    AntSet r=nearestAnts(pos,
			 0,
			 flag,
			 1);
    BOOST_CHECK_EQUAL( r.size(), 1);
    BOOST_CHECK_EQUAL( *r.begin(), 4);
  }
  pos(4,0)=2;
  {
    AntSet r=nearestAnts(pos,
			 0,
			 flag,
			 3);
    BOOST_CHECK_EQUAL( r.size(), 3);
    BOOST_CHECK(r.count(1)==1);
    BOOST_CHECK(r.count(2)==1);
    BOOST_CHECK(r.count(3)==1);
  }

  
}

BOOST_AUTO_TEST_CASE(dTdLCoeffsSingleInterpolated_isfinite_test)
{
  using namespace LibAIR;

  dTdLCoeffsSingleInterpolated c;
  boost::array<double, 4> cc = {0, 1, 2, 3};
  c.insert(0, cc, cc);

  BOOST_CHECK_EQUAL( c.isnan(), 0);  

  boost::array<double, 4> c2 = {0, NAN, 2, 3};
  c.insert(1, c2, c2);

  BOOST_CHECK_EQUAL( c.isnan(), 1);  
  
}

double fquad(double x)
{
  return x*x;
}

BOOST_AUTO_TEST_CASE(CenFD_simple)
{
  boost::array<double, 4> r=LibAIR::CenFD<4>(fquad, 0, 0.01);
  BOOST_CHECK_EQUAL( r[0], 0);   
  BOOST_CHECK_EQUAL( r[1], 0);   
  BOOST_CHECK_CLOSE( r[2], 2, 1e-6);   
  BOOST_CHECK_EQUAL( r[3], 0);   

  boost::array<double, 4> r1=LibAIR::CenFD<4>(fquad, 1, 0.01);
  BOOST_CHECK_CLOSE( r1[0], 1, 1e-6);   
  BOOST_CHECK_CLOSE( r1[1], 2, 1e-6);   
  BOOST_CHECK_CLOSE( r1[2], 2, 1e-6);   
  BOOST_CHECK_CLOSE( 1+r1[3], 1+0, 1e-6);   
}


boost::array<double, 4> f4quad(double x)
{
  boost::array<double, 4> r;
  r[0]=x*x;
  r[1]=x*x+1;
  r[2]=x*x+2*x;
  r[3]=x*x+5;
  return r;
}

BOOST_AUTO_TEST_CASE(CenFDV_simple)
{
  boost::array< boost::array<double, 4>, 3> r=LibAIR::CenFDV<3,4>(f4quad, 0, 0.01);

  BOOST_CHECK_EQUAL( r[0][0], 0);   
  BOOST_CHECK_EQUAL( r[0][1], 1);   
  BOOST_CHECK_EQUAL( r[0][3], 5);   
  BOOST_CHECK_EQUAL( r[1][0], 0);   
  BOOST_CHECK_EQUAL( r[1][1], 0);   
  BOOST_CHECK_EQUAL( r[1][2], 2);   
  BOOST_CHECK_CLOSE( r[2][0], 2, 1e-6);   
  BOOST_CHECK_CLOSE( r[2][1], 2, 1e-6);   
}
